<?php

namespace Litecms\Task\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Litecms\Task\Forms\Task as TaskForm;
use Litecms\Task\Http\Requests\TaskRequest;
use Litecms\Task\Interfaces\TaskRepositoryInterface;
use Litecms\Task\Repositories\Eloquent\Filters\TaskResourceFilter;
use Litecms\Task\Repositories\Eloquent\Presenters\TaskListPresenter;

/**
 * Resource controller class for task.
 */
class TaskResourceController extends BaseController
{

    /**
     * Initialize task resource controller.
     *
     *
     * @return null
     */
    public function __construct(TaskRepositoryInterface $task)
    {
        parent::__construct();
        $this->form = TaskForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('litecms.task.modules'), 'task', guard_url('task'));
        $this->repository = $task;
    }

    /**
     * Display a list of task.
     *
     * @return Response
     */
    public function index(TaskRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(TaskResourceFilter::class)
            ->setPresenter(TaskListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('task::task.names'))
            ->view('task::task.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display task.
     *
     * @param Request $request
     * @param Model   $task
     *
     * @return Response
     */
    public function show(TaskRequest $request, TaskRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('task::task.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('task::task.show')
            ->output();
    }

    /**
     * Show the form for creating a new task.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(TaskRequest $request, TaskRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('task::task.name'))
            ->view('task::task.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new task.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(TaskRequest $request, TaskRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('task::task.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('task/task/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/task/task'))
                ->redirect();
        }

    }

    /**
     * Show task for editing.
     *
     * @param Request $request
     * @param Model   $task
     *
     * @return Response
     */
    public function edit(TaskRequest $request, TaskRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('task::task.name'))
            ->view('task::task.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the task.
     *
     * @param Request $request
     * @param Model   $task
     *
     * @return Response
     */
    public function update(TaskRequest $request, TaskRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('task::task.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('task/task/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('task/task/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the task.
     *
     * @param Model   $task
     *
     * @return Response
     */
    public function destroy(TaskRequest $request, TaskRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('task::task.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('task/task/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('task/task/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
